# __init__.py
"""Deals with html and css documents to check their code.

Modules exported by this package:
- `html`: gets html files from a project folder, gets the HTML code
          from files, gets number of a particular element in a file
          or folder, gets elements as tags, and much more.
- `validator`: sends HTML or CSS code to the W3C Validator to check
          for errors.

"""
__version__ = "0.2.2"
